import { InMemoryRepository, InMemoryPrintHistoryRepository, InMemoryStepRepository } from '../in-memory-repository';
import { testDb } from '../../__tests__/setup';

describe('InMemoryRepository', () => {
  let repository: InMemoryRepository<{ id: number; name: string }>;

  beforeEach(() => {
    repository = new InMemoryRepository(testDb, 'test_table');
  });

  it('should create and find items', async () => {
    const item = await repository.create({ name: 'test' });
    expect(item.id).toBe(1);
    expect(item.name).toBe('test');

    const found = await repository.findById(item.id);
    expect(found).toEqual(item);
  });

  it('should return null for non-existent id', async () => {
    const found = await repository.findById(999);
    expect(found).toBeNull();
  });

  it('should update items', async () => {
    const item = await repository.create({ name: 'test' });
    const success = await repository.update(item.id, { name: 'updated' });
    expect(success).toBe(true);

    const updated = await repository.findById(item.id);
    expect(updated?.name).toBe('updated');
  });

  it('should return false when updating non-existent item', async () => {
    const success = await repository.update(999, { name: 'updated' });
    expect(success).toBe(false);
  });

  it('should delete items', async () => {
    const item = await repository.create({ name: 'test' });
    const success = await repository.delete(item.id);
    expect(success).toBe(true);

    const found = await repository.findById(item.id);
    expect(found).toBeNull();
  });

  it('should return false when deleting non-existent item', async () => {
    const success = await repository.delete(999);
    expect(success).toBe(false);
  });

  it('should list all items', async () => {
    const item1 = await repository.create({ name: 'test1' });
    const item2 = await repository.create({ name: 'test2' });

    const all = await repository.findAll();
    expect(all).toHaveLength(2);
    expect(all).toContainEqual(item1);
    expect(all).toContainEqual(item2);
  });
});

describe('InMemoryPrintHistoryRepository', () => {
  let repository: InMemoryPrintHistoryRepository;

  beforeEach(() => {
    repository = new InMemoryPrintHistoryRepository(testDb);
  });

  it('should find print history by task id', async () => {
    const history1 = await repository.create({ task_id: 1, user_id: 1, printed_at: new Date() });
    const history2 = await repository.create({ task_id: 1, user_id: 2, printed_at: new Date() });
    await repository.create({ task_id: 2, user_id: 1, printed_at: new Date() });

    const taskHistory = await repository.findByTaskId(1);
    expect(taskHistory).toHaveLength(2);
    expect(taskHistory).toContainEqual(history1);
    expect(taskHistory).toContainEqual(history2);
  });

  it('should find print history by step id', async () => {
    const history1 = await repository.create({ step_id: 1, user_id: 1, printed_at: new Date() });
    const history2 = await repository.create({ step_id: 1, user_id: 2, printed_at: new Date() });
    await repository.create({ step_id: 2, user_id: 1, printed_at: new Date() });

    const stepHistory = await repository.findByStepId(1);
    expect(stepHistory).toHaveLength(2);
    expect(stepHistory).toContainEqual(history1);
    expect(stepHistory).toContainEqual(history2);
  });
});

describe('InMemoryStepRepository', () => {
  let repository: InMemoryStepRepository;

  beforeEach(() => {
    repository = new InMemoryStepRepository(testDb);
  });

  it('should find steps by task id', async () => {
    const step1 = await repository.create({ task_id: 1, name: 'Step 1', instructions: 'Test', order: 1, print_count: 0 });
    const step2 = await repository.create({ task_id: 1, name: 'Step 2', instructions: 'Test', order: 2, print_count: 0 });
    await repository.create({ task_id: 2, name: 'Step 3', instructions: 'Test', order: 1, print_count: 0 });

    const taskSteps = await repository.findByTaskId(1);
    expect(taskSteps).toHaveLength(2);
    expect(taskSteps).toContainEqual(step1);
    expect(taskSteps).toContainEqual(step2);
  });

  it('should increment print count', async () => {
    const step = await repository.create({ task_id: 1, name: 'Step 1', instructions: 'Test', order: 1, print_count: 0 });

    const success = await repository.incrementPrintCount(step.id);
    expect(success).toBe(true);

    const updated = await repository.findById(step.id);
    expect(updated?.print_count).toBe(1);
    expect(updated?.last_printed_at).toBeInstanceOf(Date);
  });

  it('should return false when incrementing non-existent step', async () => {
    const success = await repository.incrementPrintCount(999);
    expect(success).toBe(false);
  });
});