import { testDb } from '../../__tests__/setup';
import { TaskRepository } from '../task-repository';
import { Task } from '@shared/index';

describe('TaskRepository (integration)', () => {
  let repository: TaskRepository;
  let groupId: number;

  beforeEach(async () => {
    // Clean up tasks and groups before each test
    await testDb('tasks').delete();
    await testDb('groups').delete();
    // Insert a group for foreign key
    [groupId] = await testDb('groups').insert({ name: 'Test Group' });
    repository = new TaskRepository(testDb);
  });

  afterAll(async () => {
    await testDb.destroy();
  });

  it('should create and find tasks by group id', async () => {
    const task1: Omit<Task, 'id' | 'created_at' | 'updated_at'> = {
      name: 'Task 1',
      group_id: groupId,
      print_count: 0,
      last_printed_at: undefined,
    };
    const task2: Omit<Task, 'id' | 'created_at' | 'updated_at'> = {
      name: 'Task 2',
      group_id: groupId,
      print_count: 0,
      last_printed_at: undefined,
    };
    await testDb('tasks').insert([task1, task2]);
    const found = await repository.findByGroupId(groupId);
    expect(found).toHaveLength(2);
    expect(found.map(t => t.name)).toEqual(expect.arrayContaining(['Task 1', 'Task 2']));
  });

  it('should find recent tasks', async () => {
    const now = new Date();
    const task1 = { name: 'Recent 1', group_id: groupId, print_count: 0, last_printed_at: undefined, created_at: new Date(now.getTime() - 2000).toISOString(), updated_at: new Date(now.getTime() - 2000).toISOString() };
    const task2 = { name: 'Recent 2', group_id: groupId, print_count: 0, last_printed_at: undefined, created_at: new Date(now.getTime() - 1000).toISOString(), updated_at: new Date(now.getTime() - 1000).toISOString() };
    const task3 = { name: 'Recent 3', group_id: groupId, print_count: 0, last_printed_at: undefined, created_at: now.toISOString(), updated_at: now.toISOString() };
    await testDb('tasks').insert([task1, task2, task3]);
    const found = await repository.findRecent(2);
    expect(found).toHaveLength(2);
    // Should be ordered by created_at desc, so the most recent first
    expect(found[0].name).toBe('Recent 3');
    expect(found[1].name).toBe('Recent 2');
  });

  it('should find frequent tasks', async () => {
    const task1 = { name: 'Frequent 1', group_id: groupId, print_count: 10, last_printed_at: undefined };
    const task2 = { name: 'Frequent 2', group_id: groupId, print_count: 5, last_printed_at: undefined };
    await testDb('tasks').insert([task1, task2]);
    const found = await repository.findFrequent(1);
    expect(found).toHaveLength(1);
    expect(found[0].name).toBe('Frequent 1');
  });

  it('should increment print count', async () => {
    const [id] = await testDb('tasks').insert({ name: 'Print Task', group_id: groupId, print_count: 0, last_printed_at: undefined });
    const result = await repository.incrementPrintCount(id);
    expect(result).toBe(true);
    const updated = await testDb('tasks').where({ id }).first();
    expect(updated.print_count).toBe(1);
    expect(updated.last_printed_at).toBeTruthy();
  });

  it('should return false when incrementing non-existent task', async () => {
    const result = await repository.incrementPrintCount(99999);
    expect(result).toBe(false);
  });
});