import fs from 'fs/promises';
import path from 'path';
import { Task, Step, Printer } from '@shared/index';
import { StepRepository, PrintHistoryRepository } from '../db/repositories';
import { Knex } from 'knex';
import logger from '../logger';
import { formatUtils } from './format-utils';

export class TestPrinter implements Printer {
  private readonly outputDir: string;
  private printHistoryRepo: PrintHistoryRepository;
  private stepRepository: StepRepository;

  constructor(printHistoryRepo: PrintHistoryRepository, stepRepo: StepRepository) {
    this.outputDir = path.join(process.cwd(), 'test-output');
    this.printHistoryRepo = printHistoryRepo;
    this.stepRepository = stepRepo;
    this.ensureOutputDir();
  }

  private async ensureOutputDir() {
    try {
      await fs.mkdir(this.outputDir, { recursive: true });
    } catch (error) {
      logger.error('Failed to create output directory:', error);
    }
  }

  async getTaskSteps(db: Knex, task: Task): Promise<Step[]> {
    return await this.stepRepository.findByTaskId(task.id);
  }

  async printTask(task: Task, db: Knex): Promise<void> {
    const taskSteps = await this.getTaskSteps(db, task);
    const timestamp = new Date().toISOString().replace(/[:.]/g, '-');
    const filename = path.join(this.outputDir, `task-${task.id}-${timestamp}.txt`);

    const content = [
      ...formatUtils.formatSection(`Task: ${task.name}`, ''),
      ...taskSteps.map((step, index) =>
        formatUtils.formatSection(`Step ${index + 1}: ${step.name}`, step.instructions, '-')
      ).flat(),
    ].join('\n');

    await fs.writeFile(filename, content);
    logger.info(`Printed task ${task.id} to ${filename}`);

    await this.printHistoryRepo.create({
      user_id: 1, // Replace with actual user ID if available
      task_id: task.id,
      printed_at: new Date(),
    });
  }

  async printStep(step: Step, db: Knex): Promise<void> {
    void db; // avoid unused parameter warning
    const timestamp = new Date().toISOString().replace(/[:.]/g, '-');
    const filename = path.join(this.outputDir, `step-${step.id}-${timestamp}.txt`);

    const content = formatUtils.formatSection(`Step: ${step.name}`, step.instructions).join('\n');

    await fs.writeFile(filename, content);
    logger.info(`Printed step ${step.id} to ${filename}`);

    await this.printHistoryRepo.create({
      user_id: 1, // Replace with actual user ID if available
      step_id: step.id,
      printed_at: new Date(),
    });
  }
}